function [H, G] = MakeFreqResp(N,p,q,s);
% [H, G] = MakeFreqResp(N,p,q,s);
% Make frequency responses for radwt (rational-dilation wavelet transform)
% N : length of the signal
% p,q : rational sampling factor of the lowpass channel
% s : downsampling factor of the highpass channel
%
% x ----- U(p) -- H -- D(q)-->
%    |
%     ---------- G -- D(s)-->
%
% % Example
% N = 20; p = 2; q = 3; s = 1;
% [H, G] = MakeFreqResp(N,p,q,s);

% Ilker Bayram and Ivan W. Selesnick
% Polytechnic University of New York University
% Brooklyn, NY
% 2008

% Make H

wp = (s-1)*pi/s;
ws = pi*p/q;

if rem(N,2) == 0
    % N is even
    w = 2*pi*[0:N/2-1]/N;             % frequency axis: [0, pi)
else
    % N is odd
    w = 2*pi*[0:(N-1)/2]/N;           % frequency axis: [0, pi)
end

k_pass = (abs(w) <= wp);                 % pass-band
k_stop = (abs(w) >= ws);                 % stop-band
k_trans = (abs(w) > wp) & (abs(w) < ws);    % transition-band

a = (1-1/s)*pi;
b = p/q - (1 - 1/s);
w_scaled = (abs(w) - a)/b;

H = zeros(size(w));
H(k_pass) = 1;
H(k_trans) = (1+cos(w_scaled(k_trans))) .* sqrt(2-cos(w_scaled(k_trans)))/2;


if rem(N,2) == 0
    % N is even
    H = [H 0 H(end:-1:2)];
else
    % N is odd
    H = [H H(end:-1:2)];
end

% max(abs(imag(ifft(H))))


% Make G

ws = (s-1)*pi/s;
wp = p*pi/q;

if rem(N,2) == 0
    % N is even
    w = 2*pi*[0:N/2-1]/N;             % frequency axis: [0, pi)
else
    % N is odd
    w = 2*pi*[0:(N-1)/2]/N;           % frequency axis: [0, pi)
end

k_pass = (abs(w) >= wp);                 % pass-band
k_stop = (abs(w) <= ws);                 % stop-band
k_trans = (abs(w) > ws) & (abs(w) < wp);    % transition-band

a = (1-1/s)*pi;
b = p/q - (1 - 1/s);
w_scaled = (abs(w) - a)/b;

G = zeros(size(w));
G(k_pass) = 1;
G(k_trans) = (1-cos(w_scaled(k_trans))) .* sqrt(2+cos(w_scaled(k_trans)))/2;

if rem(N,2) == 0
    % N is even
    G = [G 1 G(end:-1:2)];
else
    % N is odd
    G = [G G(end:-1:2)];
end

% max(abs(imag(ifft(G))))


% normalization

H = H*sqrt(p*q);
G = G*sqrt(s);




return

% make figures

figure(1)
clf
subplot(2,1,1)
plot(w/pi,H,w(1:p:end)/pi,G,'r')

h = ifft(H);
max(imag(h)) % should be zero
h = real(h);

g = ifft(G);
max(imag(g))  % shold be zero

subplot(2,1,2)
plot(fftshift(h))
hold on
plot(fftshift(g),'r')



